@@ -58,6 +58,7 @@ use vars qw(@ISA @EXPORT);
 use Locale::Po4a::TransTractor;
 use Locale::Po4a::Common;
 use YAML::Tiny;
+use Syntax::Keyword::Try;
 
 =head1 OPTIONS ACCEPTED BY THIS MODULE
 
@@ -122,7 +123,7 @@ my $breaks;
 =item B<debianchangelog>
 
 Handle the header and footer of
-released versions, which only contain non translatable informations.
+released versions, which only contain non translatable information.
 
 =cut
 
@@ -156,6 +157,17 @@ is provided.
 
 my %yfm_keys = ();
 
+=item B<yfm_lenient> (markdown only)
+
+Allow the YAML Front Matter parser to fail on malformated headers. This is
+particularly helpful when your file starts with a horizontal ruler instead
+of a YAML Front Matter, but you insist on using three dashes only for your
+ruler.
+
+=cut
+
+my $yfm_lenient = 0;
+
 =item B<yfm_skip_array> (markdown-only)
 
 Do not translate array values in the YAML Front Matter section.
@@ -200,6 +212,7 @@ sub initialize {
     $self->{options}{'fortunes'}        = 1;
     $self->{options}{'markdown'}        = 1;
     $self->{options}{'yfm_keys'}        = '';
+    $self->{options}{'yfm_lenient'}     = 0;
     $self->{options}{'yfm_skip_array'}  = 0;
     $self->{options}{'nobullets'}       = 0;
     $self->{options}{'keyvalue'}        = 1;
@@ -232,8 +245,9 @@ sub initialize {
 
         #        map { print STDERR "key $_\n"; } (keys %yfm_keys);
         $yfm_skip_array = $self->{options}{'yfm_skip_array'};
+        $yfm_lenient    = $self->{options}{'yfm_lenient'};
     } else {
-        foreach my $opt (qw(yfm_keys yfm_skip_array)) {
+        foreach my $opt (qw(yfm_keys yfm_lenient yfm_skip_array)) {
             die wrap_mod( "po4a::text", dgettext( "po4a", "Option %s is only valid when parsing markdown files." ),
                 $opt )
               if exists $options{$opt};
@@ -301,12 +315,12 @@ sub parse_fallback {
         if (
             $markdown
             and (
-                $line =~ /\S  $/    # explicit newline
+                $line =~ /\S  $/       # explicit newline
                 or $line =~ /"""$/
             )
           )
-        {                           # """ textblock inside macro begin
-                                    # Markdown markup needing separation _after_ this line
+        {                              # """ textblock inside macro begin
+                                       # Markdown markup needing separation _after_ this line
             $end_of_paragraph = 1;
         } else {
             undef $self->{bullet};
@@ -575,174 +589,59 @@ sub parse_markdown_bibliographic_information {
 #
 sub parse_markdown_yaml_front_matter {
     my ( $self, $line, $blockref ) = @_;
-    my ( $nextline, $nextref );
     my $yfm;
-    ( $nextline, $nextref ) = $self->shiftline();
+    my @saved_ctn;
+    my ( $nextline, $nextref ) = $self->shiftline();
+    push @saved_ctn, ( $nextline, $nextref );
     while ( defined($nextline) ) {
         last if ( $nextline =~ /^(---|\.\.\.)$/ );
         $yfm .= $nextline;
         ( $nextline, $nextref ) = $self->shiftline();
+        push @saved_ctn, ( $nextline, $nextref );
     }
-    die "Could not get the YAML Front Matter from the file." if ( length($yfm) == 0 );
-    my $yamlarray = YAML::Tiny->read_string($yfm)
-      || die "Couldn't read YAML Front Matter ($!)\n$yfm\n";
-    die "Empty YAML Front Matter" unless ( length($yamlarray) > 0 );
 
-    my ( $indent, $ctx ) = ( 0, "" );
-    foreach my $cursor (@$yamlarray) {
+    my $yamlarray;    # the parsed YFM content
+    my $yamlres;      # containing the parse error, if any
+    try {
+        $yamlarray = YAML::Tiny->read_string($yfm);
+    } catch {
+        $yamlres = $@;
+    }
 
-        # An empty document
-        if ( !defined $cursor ) {
-            $self->pushline("---\n");
+    if ( defined($yamlres) ) {
+        if ($yfm_lenient) {
+            $yamlres =~ s/ at .*$//;    # Remove the error localisation in YAML::Tiny die message, if any (for our test)
+            warn wrap_mod(
+                "po4a::text",
+                dgettext(
+                    "po4a",
+                    "Processing even if the YAML Front Matter could not be parsed. Remove the 'yfm_lenient' option for a stricter behavior.\nIgnored error: %s"
+                ),
+                $yamlres
+            );
+            my $len = ( scalar @saved_ctn ) - 1;
+            while ( $len >= 0 ) {
+                $self->unshiftline( $saved_ctn[ $len - 1 ], $saved_ctn[$len] );
 
-            # Do nothing
-
-            # A scalar document
-        } elsif ( !ref $cursor ) {
-            $self->pushline("---\n");
-            $self->pushline(
-                format_scalar( $self->translate( $cursor, $blockref, "YAML Front Matter (scalar)", "wrap" => 0 ) ) );
-
-            # A list at the root
-        } elsif ( ref $cursor eq 'ARRAY' ) {
-            if (@$cursor) {
-                $self->pushline("---\n");
-                do_array( $self, $blockref, $cursor, $indent, $ctx );
-            } else {
-                $self->pushline("---[]\n");
+                # print STDERR "Unshift ".$saved_ctn[ $len - 1] ." | ". $saved_ctn[$len] ."\n";
+                $len -= 2;
             }
-
-            # A hash at the root
-        } elsif ( ref $cursor eq 'HASH' ) {
-            if (%$cursor) {
-                $self->pushline("---\n");
-                do_hash( $self, $blockref, $cursor, $indent, $ctx );
-            } else {
-                $self->pushline("--- {}\n");
-            }
-
+            return 0;    # Not a valid YAML
         } else {
-            die( "Cannot serialize " . ref($cursor) );
-        }
-        $self->pushline("---\n");
-    }
-
-    # Escape the string to make it valid in YAML.
-    # This is very similar to YAML::Tiny::_dump_scalar but does not do the internal->UTF-8 decoding,
-    # as the translations that we feed into this function are already in UTF-8
-    sub format_scalar {
-        my $string = $_[0];
-        my $is_key = $_[1];
-
-        return '~'  unless defined $string;
-        return "''" unless length $string;
-        if ( Scalar::Util::looks_like_number($string) ) {
-
-            # keys and values that have been used as strings get quoted
-            if ($is_key) {
-                return qq['$string'];
-            } else {
-                return $string;
-            }
-        }
-        if ( $string =~ /[\\\'\n]/ ) {
-            $string =~ s/\\/\\\\/g;
-            $string =~ s/"/\\"/g;
-            $string =~ s/\n/\\n/g;
-            return qq|"$string"|;
-        }
-        if ( $string =~ /(?:^[~!@#%&*|>?:,'"`{}\[\]]|^-+$|\s|:\z)/ ) {
-            return "'$string'";
-        }
-        return $string;
-    }
-
-    sub do_array {
-        my ( $self, $blockref, $array, $indent, $ctx ) = @_;
-        foreach my $el (@$array) {
-            my $header = ( '  ' x $indent ) . '- ';
-            my $type   = ref $el;
-            if ( !$type ) {
-                if ($yfm_skip_array) {
-                    $self->pushline( $header . YAML::Tiny::_dump_scalar( "dummy", $el, 0 ) . "\n" );
-                } else {
-                    $self->pushline( $header
-                          . format_scalar( $self->translate( $el, $blockref, "YAML Front Matter:$ctx", "wrap" => 0 ) )
-                          . "\n" );
-                }
-
-            } elsif ( $type eq 'ARRAY' ) {
-                if (@$el) {
-                    $self->pushline( $header . "\n" );
-                    do_array( $self, $blockref, $el, $indent + 1, $ctx );
-                } else {
-                    $self->pushline( $header . " []\n" );
-                }
-
-            } elsif ( $type eq 'HASH' ) {
-                if ( keys %$el ) {
-                    $self->pushline( $header . "\n" );
-                    do_hash( $self, $blockref, $el, $indent + 1, $ctx );
-                } else {
-                    $self->pushline( $header . " {}\n" );
-                }
-
-            } else {
-                die "YAML $type references not supported";
-            }
+            die wrap_mod(
+                "po4a::text",
+                dgettext(
+                    "po4a",
+                    "Could not get the YAML Front Matter from the file. If you did not intend to add a YAML front matter "
+                      . "but an horizontal ruler, please use '----' instead, or pass the 'yfm_lenient' option.\nError: %s\nContent of the YFM: %s"
+                ),
+                $yamlres, $yfm
+            );
         }
     }
 
-    sub do_hash {
-        my ( $self, $blockref, $hash, $indent, $ctx ) = @_;
-        foreach my $name ( sort keys %$hash ) {
-            my $el     = $hash->{$name};
-            my $header = ( '  ' x $indent ) . YAML::Tiny::_dump_scalar( "dummy", $name, 1 ) . ":";
-            my $type   = ref $el;
-            if ( !$type ) {
-                if ( ( not %yfm_keys ) || $yfm_keys{$name} )
-                {    # either no key is provided, or the key we need is also provided
-                    $self->pushline(
-                        $header . ' '
-                          . format_scalar(
-                            $self->translate( $el, $blockref, "YAML Front Matter:$ctx $name", "wrap" => 0 )
-                          )
-                          . "\n"
-                    );
-                } else {
-
-                    # Work around a bug in YAML::Tiny that quotes numbers
-                    # See https://github.com/Perl-Toolchain-Gang/YAML-Tiny#additional-perl-specific-notes
-                    if ( Scalar::Util::looks_like_number($el) ) {
-                        $self->pushline("$header $el\n");
-                    } else {
-                        $self->pushline( $header . ' ' . YAML::Tiny::_dump_scalar( "dummy", $el ) . "\n" );
-                    }
-                }
-
-            } elsif ( $type eq 'ARRAY' ) {
-                if (@$el) {
-                    $self->pushline( $header . "\n" );
-                    do_array( $self, $blockref, $el, $indent + 1, "$ctx $name" );
-                } else {
-                    $self->pushline( $header . " []\n" );
-                }
-
-            } elsif ( $type eq 'HASH' ) {
-                if ( keys %$el ) {
-                    $self->pushline( $header . "\n" );
-                    do_hash( $self, $blockref, $el, $indent + 1, "$ctx $name" );
-                } else {
-                    $self->pushline( $header . " {}\n" );
-                }
-
-            } else {
-                die "YAML $type references not supported";
-            }
-        }
-    }
-
-    return;
+    $self->handle_yaml( $blockref, $yamlarray, \%yfm_keys, $yfm_skip_array );
+    return 1;    # Valid YAML
 }
 
 sub parse_markdown {
@@ -757,8 +656,11 @@ sub parse_markdown {
             parse_markdown_bibliographic_information( $self, $line, $ref );
             return ( $paragraph, $wrapped_mode, $expect_header, $end_of_paragraph );
         } elsif ( $line =~ /^---$/ ) {
-            parse_markdown_yaml_front_matter( $self, $line, $ref );
-            return ( $paragraph, $wrapped_mode, $expect_header, $end_of_paragraph );
+            if ( parse_markdown_yaml_front_matter( $self, $line, $ref ) ) {    # successfully parsed
+                return ( $paragraph, $wrapped_mode, $expect_header, $end_of_paragraph );
+            }
+
+            # If it wasn't a YFM paragraph after all, stop expecting a header and keep going
         }
     }
     if (    ( $line =~ m/^(={4,}|-{4,})$/ )
@@ -838,11 +740,11 @@ sub parse_markdown {
         $paragraph        = "";
         $end_of_paragraph = 1;
     } elsif (
-        $line =~ /^\s*\[\[\!\S+\s*$/    # macro begin
+        $line =~ /^\s*\[\[\!\S+\s*$/       # macro begin
         or $line =~ /^\s*"""\s*\]\]\s*$/
       )
-    {                                   # """ textblock inside macro end
-                                        # Avoid translating Markdown lines containing only markup
+    {                                      # """ textblock inside macro end
+                                           # Avoid translating Markdown lines containing only markup
         do_paragraph( $self, $paragraph, $wrapped_mode );
         $paragraph    = "";
         $wrapped_mode = $defaultwrap;
@@ -853,8 +755,8 @@ sub parse_markdown {
         $paragraph        = "$line\n";
         $wrapped_mode     = 0;
         $end_of_paragraph = 1;
-    } elsif ( $line =~ /^"""/ ) {                          # """ textblock inside macro end
-                                                           # Markdown markup needing separation _before_ this line
+    } elsif ( $line =~ /^"""/ ) {    # """ textblock inside macro end
+                                     # Markdown markup needing separation _before_ this line
         do_paragraph( $self, $paragraph, $wrapped_mode );
         $paragraph    = "$line\n";
         $wrapped_mode = $defaultwrap;
@@ -907,7 +809,7 @@ sub parse {
             # Some Markdown markup can (or might) not survive wrapping
             $wrapped_mode = 0
               if (
-                $paragraph =~ /^>/ms                        # blockquote
+                $paragraph    =~ /^>/ms                     # blockquote
                 or $paragraph =~ /^( {8}|\t)/ms             # monospaced
                 or $paragraph =~ /^\$(\S+[{}]\S*\s*)+/ms    # Xapian macro
                 or $paragraph =~ /<(?![a-z]+[:@])/ms        # maybe html (tags but not wiki <URI>)
@@ -978,7 +880,7 @@ sub do_paragraph {
             if ( $text !~ m/\S[ \t][ \t][ \t]+\S/s ) {
                 my $bullet_regex = quotemeta( $indent1 . $bullet );
                 $bullet_regex =~ s/[0-9]+/\\d\+/;
-                if ( $para eq ''
+                if (   $para eq ''
                     or $para =~ m/^(\s*)((?:[-*o+]|([0-9]+[.\)])|\([0-9]+\))\s+)([^\n]*\n)(.*)$/s
                     or $para =~ m/^$bullet_regex\S/s )
                 {
@@ -1038,16 +940,20 @@ Tested successfully on simple text files and NEWS.Debian files.
 
 =head1 AUTHORS
 
- Nicolas François <nicolas.francois@centraliens.net>
+ Nicolas Francois <nicolas.francois@centraliens.net>
 
 =head1 COPYRIGHT AND LICENSE
 
- Copyright © 2005-2008 Nicolas FRANÇOIS <nicolas.francois@centraliens.net>.
+ Copyright (c) 2005-2008 Nicolas FRANCOIS <nicolas.francois@centraliens.net>.
 
- Copyright © 2008-2009, 2018 Jonas Smedegaard <dr@jones.dk>.
- Copyright © 2020 Martin Quinson <mquinson#debian.org>.
+ Copyright (c) 2008-2009, 2018 Jonas Smedegaard <dr@jones.dk>.
+ Copyright (c) 2020 Martin Quinson <mquinson#debian.org>.
 
 This program is free software; you may redistribute it and/or modify it
 under the terms of GPL (see the COPYING file).
 
 =cut
+
+__END__
+
+#  LocalWords: Charset charset po UTF gettext msgid nostrip GPL
